// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

abstract class AuthenticationFailureInterface implements Exception {
  String code;
  String message;

  AuthenticationFailureInterface(this.code, this.message);
  AuthenticationFailureInterface.fromCode(this.code)
      : message = 'An unknown error occurred.';
}

/// {@template apply_action_code_failure}
/// Thrown if during the apply action code process if a failure occurs.
/// {@endtemplate}
abstract class ApplyActionCodeFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro apply_action_code_failure}
  ApplyActionCodeFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro apply_action_code_failure}
  ApplyActionCodeFailureInterface.fromCode(String code) : super.fromCode(code);
}

/// {@template sign_up_with_email_and_password_failure}
/// Thrown if during the sign up process if a failure occurs.
/// {@endtemplate}
abstract class SignUpWithEmailAndPasswordFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_up_with_email_and_password_failure}
  SignUpWithEmailAndPasswordFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_up_with_email_and_password_failure}
  SignUpWithEmailAndPasswordFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template fetch_sign_in_methods_failure}
/// Thrown during the fetch sign in methods if a failure occurs.
/// {@endtemplate}
abstract class FetchSignInMethodsForEmailFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro fetch_sign_in_methods_failure}
  FetchSignInMethodsForEmailFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro fetch_sign_in_methods_failure}
  FetchSignInMethodsForEmailFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template sign_in_with_credential_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithCredentialFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_credential_failure}
  SignInWithCredentialFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_credential_failure}
  SignInWithCredentialFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template sign_in_anonymously_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInAnonymouslyFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_anonymously_failure}
  SignInAnonymouslyFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_anonymously_failure}
  SignInAnonymouslyFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template sign_in_with_google_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithGoogleFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_google_failure}
  SignInWithGoogleFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_google_failure}
  SignInWithGoogleFailureInterface.fromCode(String code) : super.fromCode(code);
}

/// {@template sign_in_with_facebook_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithFacebookFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_facebook_failure}
  SignInWithFacebookFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_facebook_failure}
  SignInWithFacebookFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template sign_in_with_apple_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithAppleFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_apple_failure}
  SignInWithAppleFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_apple_failure}
  SignInWithAppleFailureInterface.fromCode(String code) : super.fromCode(code);
}

/// {@template sign_in_with_twitter_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithTwitterFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_twitter_failure}
  SignInWithTwitterFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_twitter_failure}
  SignInWithTwitterFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template sign_in_with_email_link_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithEmailLinkFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_email_link_failure}
  SignInWithEmailLinkFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_email_link_failure}
  SignInWithEmailLinkFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template sign_in_with_email_and_password_failure}
/// Thrown during the sign in process if a failure occurs.
/// {@endtemplate}
abstract class SignInWithEmailAndPasswordFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro sign_in_with_email_and_password_failure}
  SignInWithEmailAndPasswordFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro sign_in_with_email_and_password_failure}
  SignInWithEmailAndPasswordFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template send_email_verification_failure}
/// Thrown during the email verification process if a failure occurs.
/// {@endtemplate}
abstract class SendEmailVerificationFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro send_email_verification_failure}
  SendEmailVerificationFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro send_email_verification_failure}
  SendEmailVerificationFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template send_password_reset_email_failure}
/// Thrown during the password reset process if a failure occurs.
/// {@endtemplate}
abstract class SendPasswordResetEmailFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro send_password_reset_email_failure}
  SendPasswordResetEmailFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro send_password_reset_email_failure}
  SendPasswordResetEmailFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template send_sign_in_link_email_failure}
/// Thrown during the sign in link process if a failure occurs.
/// {@endtemplate}
abstract class SendSignInLinkEmailFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro send_sign_in_link_email_failure}
  SendSignInLinkEmailFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro send_sign_in_link_email_failure}
  SendSignInLinkEmailFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template confirm_password_reset_failure}
/// Thrown during the password reset process if a failure occurs.
/// {@endtemplate}
abstract class ConfirmPasswordResetFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro confirm_password_reset_failure}
  ConfirmPasswordResetFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro confirm_password_reset_failure}
  ConfirmPasswordResetFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template verify_password_reset_code_failure}
/// Thrown during the password reset process if a failure occurs.
/// {@endtemplate}
abstract class VerifyPasswordResetCodeFailureInterface
    extends AuthenticationFailureInterface {
  /// {@macro verify_password_reset_code_failure}
  VerifyPasswordResetCodeFailureInterface(String code, String message)
      : super(code, message);

  /// {@macro verify_password_reset_code_failure}
  VerifyPasswordResetCodeFailureInterface.fromCode(String code)
      : super.fromCode(code);
}

/// {@template refresh_failure}
/// Thrown during the refresh process if a failure occurs.
/// {@endtemplate}
abstract class RefreshFailureInterface extends AuthenticationFailureInterface {
  /// {@macro refresh_failure}
  RefreshFailureInterface(String code, String message) : super(code, message);

  /// {@macro refresh_failure}
  RefreshFailureInterface.fromCode(String code) : super.fromCode(code);
}

/// {@template sign_out_failure}
/// Thrown during the sign out process if a failure occurs.
/// {@endtemplate}
abstract class SignOutFailureInterface extends AuthenticationFailureInterface {
  /// {@macro sign_out_failure}
  SignOutFailureInterface(String code, String message) : super(code, message);

  /// {@macro sign_out_failure}
  SignOutFailureInterface.fromCode(String code) : super.fromCode(code);
}
