// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'crud_cubit.dart';

enum CrudStatus {
  idle,
  loading,
  success,
  failure,
}

class CrudState<T> extends Equatable {
  final CrudStatus status;
  final List<T?> data;
  final Stream<List<T?>>? stream;
  final String? errorMessage;

  const CrudState({
    this.status = CrudStatus.idle,
    this.data = const [],
    this.stream,
    this.errorMessage,
  });

  @override
  List<Object?> get props => [status, data];

  CrudState<T> copyWith({
    CrudStatus? status,
    List<T?>? data,
    Stream<List<T?>>? stream,
    String? errorMessage,
  }) {
    return CrudState<T>(
      status: status ?? this.status,
      data: data ?? this.data,
      stream: stream ?? this.stream,
      errorMessage: errorMessage ?? this.errorMessage,
    );
  }

  @override
  String toString() =>
      // ignore: lines_longer_than_80_chars
      'CrudState(status: $status, data: $data, stream: ${stream != null ? 'listening' : 'null'}, errorMessage: $errorMessage)';
}
